@@ -1,5 +1,6 @@
 import time
 import inspect
+from typing import Callable, Dict, List
 import uuid
 import atexit
 import asyncio
@@ -7,12 +8,13 @@ import warnings
 
 import requests
 
-from flask import current_app, request, jsonify, abort
+from flask import Flask, Response, current_app, request, jsonify, abort
 
 from nacl.exceptions import BadSignatureError
 from nacl.signing import VerifyKey
 
 from flask_discord_interactions.models.autocomplete import AutocompleteResult
+from flask_discord_interactions.models.option import Option
 
 try:
     import aiohttp
@@ -21,7 +23,8 @@ except ImportError:
 
 from flask_discord_interactions.command import Command, SlashCommandGroup
 from flask_discord_interactions.context import Context, ApplicationCommandType
-from flask_discord_interactions.models import Message, Modal, ResponseType
+from flask_discord_interactions.models import Message, Modal, ResponseType, Permission
+from flask_discord_interactions.utils import static_or_instance
 
 
 class InteractionType:
@@ -46,142 +49,119 @@ class DiscordInteractionsBlueprint:
 
     def add_command(
         self,
-        command,
-        name=None,
-        description=None,
-        options=None,
-        annotations=None,
-        type=ApplicationCommandType.CHAT_INPUT,
-        default_permission=None,
-        default_member_permissions=None,
-        dm_permission=None,
-        permissions=None,
-        name_localizations=None,
-        description_localizations=None,
+        command: Callable,
+        name: str = None,
+        description: str = None,
+        *,
+        options: List[Option] = None,
+        annotations: Dict[str, str] = None,
+        type: int = ApplicationCommandType.CHAT_INPUT,
+        default_member_permissions: int = None,
+        dm_permission: bool = None,
+        name_localizations: Dict[str, str] = None,
+        description_localizations: Dict[str, str] = None,
     ):
         """
         Create and add a new :class:`ApplicationCommand`.
 
         Parameters
         ----------
-        command
+        command: Callable
             Function to execute when the command is run.
-        name
+        name: str
             The name of the command, as displayed in the Discord client.
-        name_localizations
+        name_localizations: Dict[str, str]
             A dictionary of localizations for the name of the command.
-        description
+        description: str
             The description of the command.
-        description_localizations
+        description_localizations: Dict[str, str]
             A dictionary of localizations for the description of the command.
-        options
+        options: List[Option]
             A list of options for the command, overriding the function's
             keyword arguments.
-        annotations
+        annotations: Dict[str, str]
             If ``options`` is not provided, descriptions for each of the
             options defined in the function's keyword arguments.
-        type
-            The ``ApplicationCommandType`` of the command.
-        default_permission
-            Deprecated as of v1.5! Whether the command is enabled by default.
-        default_member_permissions
-            A permission integer defining the required permissions a user must have to run the command
-        dm_permission
-            Indicates whether the command can be used in DMs
-        permissions
-            List of permission overwrites.
+        type: int
+            The class:`.ApplicationCommandType` of the command.
+        default_member_permissions: int
+            A permission integer defining the required permissions a user must have to run the command.
+        dm_permission: bool
+            Indicates whether the command can be used in DMs.
         """
         command = Command(
-            command,
-            name,
-            description,
-            options,
-            annotations,
-            type,
-            default_permission,
-            default_member_permissions,
-            dm_permission,
-            permissions,
-            name_localizations,
-            description_localizations,
-            self,
+            command=command,
+            name=name,
+            description=description,
+            options=options,
+            annotations=annotations,
+            type=type,
+            default_member_permissions=default_member_permissions,
+            dm_permission=dm_permission,
+            name_localizations=name_localizations,
+            description_localizations=description_localizations,
+            discord=self,
         )
         self.discord_commands[command.name] = command
         return command
 
-    def add_slash_command(self, *args, **kwargs):
-        """
-        Deprecated! As of v1.1.0, ``add_slash_command`` has been renamed to
-        :meth:`add_command`, as it can now add User and Message commands.
-        """
-        warnings.warn(
-            "Deprecated! As of v1.1.0, add_slash_command has been renamed to "
-            "add_command, as it can now add User and Message commands.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        return self.add_command(*args, **kwargs)
-
     def command(
         self,
-        name=None,
-        description=None,
-        options=None,
-        annotations=None,
-        type=ApplicationCommandType.CHAT_INPUT,
-        default_permission=None,
-        default_member_permissions=None,
-        dm_permission=None,
-        permissions=None,
-        name_localizations=None,
-        description_localizations=None,
+        name: str = None,
+        description: str = None,
+        *,
+        options: List[Option] = None,
+        annotations: Dict[str, str] = None,
+        type: int = ApplicationCommandType.CHAT_INPUT,
+        default_member_permissions: int = None,
+        dm_permission: bool = None,
+        name_localizations: Dict[str, str] = None,
+        description_localizations: Dict[str, str] = None,
     ):
         """
         Decorator to create a new :class:`Command`.
 
         Parameters
         ----------
-        name
+        name: str
             The name of the command, as displayed in the Discord client.
-        name_localizations
+        name_localizations: Dict[str, str]
             A dictionary of localizations for the name of the command.
-        description
+        description: str
             The description of the command.
-        description_localizations
+        description_localizations: Dict[str, str]
             A dictionary of localizations for the description of the command.
-        options
+        options: List[Option]
             A list of options for the command, overriding the function's
             keyword arguments.
-        annotations
+        annotations: Dict[str, str]
             If ``options`` is not provided, descriptions for each of the
             options defined in the function's keyword arguments.
-        type
+        type: int
             The ``ApplicationCommandType`` of the command.
-        default_permission
-            Deprecated as of v1.5! Whether the command is enabled by default.
-        default_member_permissions
+        default_member_permissions: int
             A permission integer defining the required permissions a user must have to run the command
-        dm_permission
+        dm_permission: bool
             Indicates whether the command can be used in DMs
-        permissions
-            List of permission overwrites.
+
+        Returns
+        -------
+        Callable[Callable, Command]
         """
 
         def decorator(func):
             nonlocal name, description, type, options
             command = self.add_command(
                 func,
-                name,
-                description,
-                options,
-                annotations,
-                type,
-                default_permission,
-                default_member_permissions,
-                dm_permission,
-                permissions,
-                name_localizations,
-                description_localizations,
+                name=name,
+                description=description,
+                options=options,
+                annotations=annotations,
+                type=type,
+                default_member_permissions=default_member_permissions,
+                dm_permission=dm_permission,
+                name_localizations=name_localizations,
+                description_localizations=description_localizations,
             )
             return command
 
@@ -189,15 +169,14 @@ class DiscordInteractionsBlueprint:
 
     def command_group(
         self,
-        name,
-        description="No description",
-        is_async=False,
-        default_permission=None,
-        default_member_permissions=None,
-        dm_permission=None,
-        permissions=None,
-        name_localizations=None,
-        description_localizations=None,
+        name: str,
+        description: str = "No description",
+        *,
+        is_async: bool = False,
+        default_member_permissions: int = None,
+        dm_permission: bool = None,
+        name_localizations: Dict[str, str] = None,
+        description_localizations: Dict[str, str] = None,
     ):
         """
         Create a new :class:`SlashCommandGroup`
@@ -205,50 +184,49 @@ class DiscordInteractionsBlueprint:
 
         Parameters
         ----------
-        name
+        name: str
             The name of the command group, as displayed in the Discord client.
-        name_localizations
+        name_localizations: Dict[str, str]
             A dictionary of localizations for the name of the command group.
-        description
+        description: str
             The description of the command group.
-        description_localizations
+        description_localizations: Dict[str, str]
             A dictionary of localizations for the description of the command group.
-        is_async
+        is_async: bool
             Whether the subgroup should be considered async (if subcommands
             get an :class:`.AsyncContext` instead of a :class:`Context`.)
-        default_permission
-            Deprecated as of v1.5! Whether the command is enabled by default.
-        default_member_permissions
+        default_member_permissions: int
             A permission integer defining the required permissions a user must have to run the command
-        dm_permission
+        dm_permission: bool
             Indicates whether the command canbe used in DMs
-        permissions
-            List of permission overwrites. These apply to the entire group.
+
+        Returns
+        -------
+        SlashCommandGroup
+            The newly created command group.
         """
 
         group = SlashCommandGroup(
-            name,
-            description,
-            is_async,
-            default_permission,
-            default_member_permissions,
-            dm_permission,
-            permissions,
-            name_localizations,
-            description_localizations,
+            name=name,
+            description=description,
+            is_async=is_async,
+            default_member_permissions=default_member_permissions,
+            dm_permission=dm_permission,
+            name_localizations=name_localizations,
+            description_localizations=description_localizations,
         )
         self.discord_commands[name] = group
         return group
 
-    def add_custom_handler(self, handler, custom_id=None):
+    def add_custom_handler(self, handler: Callable, custom_id: str = None):
         """
         Add a handler for an incoming interaction with the specified custom ID.
 
         Parameters
         ----------
-        handler
+        handler: Callable
             The function to call to handle the incoming interaction.
-        custom_id
+        custom_id: str
             The custom ID to respond to. If not specified, the ID will be
             generated randomly.
 
@@ -263,7 +241,7 @@ class DiscordInteractionsBlueprint:
         self.custom_id_handlers[custom_id] = handler
         return custom_id
 
-    def custom_handler(self, custom_id=None):
+    def custom_handler(self, custom_id: str = None):
         """
         Returns a decorator to register a handler for a custom ID.
 
@@ -272,6 +250,10 @@ class DiscordInteractionsBlueprint:
         custom_id
             The custom ID to respond to. If not specified, the ID will be
             generated randomly.
+
+        Returns
+        -------
+        Callable[Callable, str]
         """
 
         def decorator(func):
@@ -281,15 +263,15 @@ class DiscordInteractionsBlueprint:
 
         return decorator
 
-    def add_autocomplete_handler(self, handler, command_name):
+    def add_autocomplete_handler(self, handler: Callable, command_name: str):
         """
         Add a handler for an incoming autocomplete request.
 
         Parameters
         ----------
-        handler
+        handler: Callable
             The function to call to handle the incoming autocomplete request.
-        command_name
+        command_name: str
             The name of the command to autocomplete.
         """
         self.autocomplete_handlers[command_name] = handler
@@ -300,31 +282,36 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
     Handles registering a collection of :class:`Command` s, receiving
     incoming interaction data, and sending/editing/deleting messages via
     webhook.
+
+    Parameters
+    ----------
+    app: Flask
+        The Flask application to bind to.
     """
 
-    def __init__(self, app=None):
+    def __init__(self, app: Flask = None):
         super().__init__()
 
         self.app = app
         if app is not None:
             self.init_app(app)
 
-    def init_app(self, app):
+    def init_app(self, app: Flask):
         """
         Initialize a Flask app with Discord-specific configuration and
         attributes.
 
         Parameters
         ----------
-        app
+        app: Flask
             The Flask app to initialize.
         """
 
-        app.config.setdefault("DISCORD_BASE_URL", "https://discord.com/api/v9")
+        app.config.setdefault("DISCORD_BASE_URL", "https://discord.com/api/v10")
         app.config.setdefault("DISCORD_CLIENT_ID", "")
         app.config.setdefault("DISCORD_PUBLIC_KEY", "")
         app.config.setdefault("DISCORD_CLIENT_SECRET", "")
-        app.config.setdefault("DISCORD_SCOPE", "applications.commands.update applications.commands.permissions.update")
+        app.config.setdefault("DISCORD_SCOPE", "applications.commands.update")
         app.config.setdefault("DONT_VALIDATE_SIGNATURE", False)
         app.config.setdefault("DONT_REGISTER_WITH_DISCORD", False)
         app.discord_commands = self.discord_commands
@@ -332,7 +319,8 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
         app.autocomplete_handlers = self.autocomplete_handlers
         app.discord_token = None
 
-    def fetch_token(self, app=None):
+    @static_or_instance
+    def fetch_token(self, app: Flask = None):
         """
         Fetch an OAuth2 token from Discord using the ``CLIENT_ID`` and
         ``CLIENT_SECRET`` with the ``applications.commands.update`` scope. This
@@ -358,6 +346,7 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
                 time.time() + app.discord_token["expires_in"] / 2
             )
             return
+
         response = requests.post(
             app.config["DISCORD_BASE_URL"] + "/oauth2/token",
             data={
@@ -374,22 +363,28 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
             time.time() + app.discord_token["expires_in"] / 2
         )
 
-    def auth_headers(self, app):
+    @staticmethod
+    def auth_headers(app: Flask):
         """
         Get the Authorization header required for HTTP requests to the
         Discord API.
 
         Parameters
         ----------
-        app
+        app: Flask
             The Flask app with the relevant access token.
+
+        Returns
+        -------
+        Dict[str, str]
+            The Authorization header.
         """
 
         if app.discord_token is None or time.time() > app.discord_token["expires_on"]:
-            self.fetch_token(app)
+            DiscordInteractions.fetch_token(app)
         return {"Authorization": f"Bearer {app.discord_token['access_token']}"}
 
-    def update_commands(self, app=None, guild_id=None):
+    def update_commands(self, app: Flask = None, guild_id: str = None):
         """
         Update the list of commands registered with Discord.
         This method will overwrite all existing commands.
@@ -401,9 +396,9 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
 
         Parameters
         ----------
-        app
+        app: Flask
             The Flask app with the relevant Discord access token.
-        guild_id
+        guild_id: str
             The ID of the Discord guild to register commands to. If omitted,
             the commands are registered globally.
         """
@@ -447,43 +442,208 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
             for command in app.discord_commands.values():
                 command.id = command.name
 
-        if guild_id:
-            for command in app.discord_commands.values():
-                if (
-                    not app.config["DONT_REGISTER_WITH_DISCORD"]
-                    and command.permissions is not None
-                ):
-                    response = requests.put(
-                        url + "/" + command.id + "/permissions",
-                        json={"permissions": command.dump_permissions()},
-                        headers=self.auth_headers(app),
-                    )
-
-                    try:
-                        response.raise_for_status()
-                    except requests.exceptions.HTTPError:
-                        raise ValueError(
-                            f"Unable to register permissions for {command.id}:"
-                            f"{response.status_code} {response.text}"
-                        )
-
-                    self.throttle(response)
-
-    def update_slash_commands(self, *args, **kwargs):
+    @staticmethod
+    def build_permission_overwrite_url(
+        self,
+        command: Command = None,
+        *,
+        guild_id: str,
+        command_id: str = None,
+        token: str = None,
+        app: Flask = None,
+        application_id: str = None,
+        base_url: str = None,
+    ):
         """
-        Deprecated! As of v1.1.0, ``update_slash_commands`` has been renamed to
-        ``update_commands``, as it updates User and Message commands as well.
+        Build the URL for getting or setting permission overwrites for a
+        specific guild and command.
         """
-        warnings.warn(
-            "Deprecated! As of v1.1.0, update_slash_commands has been renamed "
-            "to update_commands, as it updates User and Message commands too.",
-            DeprecationWarning,
-            stacklevel=2,
+        if not app and self and self.app:
+            app = self.app
+
+        if not application_id or not base_url:
+            if app:
+                application_id = app.config["DISCORD_CLIENT_ID"]
+                base_url = app.config["DISCORD_BASE_URL"]
+            else:
+                raise ValueError(
+                    "This method requires the application ID and base URL."
+                    " Either provide these as arguments or provide an app "
+                    " instance with the relevant configuration."
+                )
+
+        if command_id is None:
+            if command:
+                command_id = command.id
+            else:
+                raise ValueError(
+                    "You must supply either a command ID or a Command instance."
+                )
+
+        url = f"{base_url}/applications/{application_id}/guilds/{guild_id}/commands/{command_id}/permissions"
+        auth = (
+            {"Authorization": f"Bearer {token}"}
+            if token
+            else DiscordInteractions.auth_headers(app)
         )
 
-        return self.update_commands(*args, **kwargs)
+        return url, auth
 
-    def throttle(self, response):
+    @static_or_instance
+    def get_permission_overwrites(
+        self,
+        command: Command = None,
+        *,
+        guild_id: str,
+        command_id: str = None,
+        token: str = None,
+        app: Flask = None,
+        application_id: str = None,
+        base_url: str = None,
+    ):
+        """
+        Get the list of permission overwrites in a specific guild for a
+        specific command. You must supply a Bearer token from a user with
+        "Manage Roles" and "Manage Server" permissions in the given guild.
+
+        If the token is omitted, the bot's token will be used. Note that this
+        only works if the bot's developer account is an admin in the guild.
+        This is handy for small bots on your own servers, but you shouldn't
+        rely on this for anything you want others to use in their servers.
+
+        There are a many ways to call this method, here are a few:
+
+        .. code-block:: python
+
+            # Without the app or instance, useful in a background worker
+            DiscordInteractions.get_permission_overwrites(
+                guild_id=...,
+                command_id=...,
+                token=...,
+                application_id=...,
+                base_url=...,
+            )
+
+            # With the instance and app passed in, useful in an app-factory project
+            discord.get_permission_overwrites(
+                guild_id=...,
+                command=...,
+                token=...,
+                app=...,
+            )
+
+            # With the instance and a bound app, using an implicit token
+            # useful in most small projects
+            discord.get_permission_overwrites(
+                guild_id=...,
+                command=...,
+            )
+
+        Parameters
+        ----------
+        command: Command
+            The :class:`.Command` to retrieve permissions for.
+        guild_id: str
+            The ID of the guild to retrieve permissions from.
+        command_id: str
+            The ID of the command to retrieve permissions for.
+        token: str
+            A bearer token from an admin of the guild (not including the
+            leading ``Bearer`` word). If omitted, the bot's token will be
+            used instead.
+        app: Flask
+            The Flask app with the relevant Discord application ID.
+        application_id: str
+            The ID of the Discord application to retrieve permissions from.
+        base_url: str
+            The base URL of the Discord API.
+
+        Returns
+        -------
+        List[Permission]
+            A list of permission overwrites for the given command.
+        """
+
+        url, auth = DiscordInteractions.build_permission_overwrite_url(
+            self,
+            command,
+            guild_id=guild_id,
+            command_id=command_id,
+            token=token,
+            app=app,
+            application_id=application_id,
+            base_url=base_url,
+        )
+
+        response = requests.get(
+            url,
+            headers=auth,
+        )
+        response.raise_for_status()
+
+        return [Permission.from_dict(perm) for perm in response.json()]
+
+    @static_or_instance
+    def set_permission_overwrites(
+        self,
+        permissions: List[Permission],
+        command: Command = None,
+        *,
+        guild_id: str,
+        command_id: str = None,
+        token: str = None,
+        app: Flask = None,
+        application_id: str = None,
+        base_url: str = None,
+    ):
+        """
+        Overwrite the list of permission overwrites in a specific guild for a
+        specific command. You must supply a Bearer token from a user with
+        "Manage Roles" and "Manage Server" permissions in the given guild.
+
+        This method requires access to the application ID and base URL. For
+        convenience, it can be called as either an instance method (using the
+        bound app's configuration) or a static method.
+
+        Parameters
+        ----------
+        command: Command
+            The :class:`.Command` to retrieve permissions for.
+        guild_id: str
+            The ID of the guild to retrieve permissions from.
+        command_id: str
+            The ID of the command to retrieve permissions for.
+        token: str
+            A bearer token from an admin of the guild (not including the
+            leading ``Bearer`` word). If omitted, the bot's token will be
+            used instead.
+        app: Flask
+            The Flask app with the relevant Discord application ID.
+        application_id: str
+            The ID of the Discord application to retrieve permissions from.
+        base_url: str
+            The base URL of the Discord API.
+        """
+
+        url, auth = DiscordInteractions.build_permission_overwrite_url(
+            self,
+            command,
+            guild_id=guild_id,
+            command_id=command_id,
+            token=token,
+            app=app,
+            application_id=application_id,
+            base_url=base_url,
+        )
+
+        response = requests.put(
+            url,
+            headers=auth,
+            json={"permissions": [perm.dump() for perm in permissions]},
+        )
+        response.raise_for_status()
+
+    def throttle(self, response: requests.Response):
         """
         Throttle the number of HTTP requests made to Discord
         using the ``X-RateLimit`` headers
@@ -491,7 +651,7 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
 
         Parameters
         ----------
-        response
+        response: requests.Response
             Response object from a previous HTTP request
         """
 
@@ -507,7 +667,9 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
             )
             time.sleep(max(wait_time, 0))
 
-    def register_blueprint(self, blueprint, app=None):
+    def register_blueprint(
+        self, blueprint: DiscordInteractionsBlueprint, app: Flask = None
+    ):
         """
         Register a :class:`DiscordInteractionsBlueprint` to this
         DiscordInteractions class. Updates this instance's list of
@@ -516,10 +678,10 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
 
         Parameters
         ----------
-        blueprint
+        blueprint: DiscordInteractionsBlueprint
             The :class:`DiscordInteractionsBlueprint` to add
             :class:`Command` s from.
-        app
+        app: Flask
             The Flask app with the relevant Discord commands.
         """
 
@@ -530,7 +692,7 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
         app.custom_id_handlers.update(blueprint.custom_id_handlers)
         app.autocomplete_handlers.update(blueprint.autocomplete_handlers)
 
-    def run_command(self, data):
+    def run_command(self, data: dict):
         """
         Run the corresponding :class:`Command` given incoming interaction
         data.
@@ -539,6 +701,11 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
         ----------
         data
             Incoming interaction data.
+
+        Returns
+        -------
+        Message
+            The resulting message from the command.
         """
 
         command_name = data["data"]["name"]
@@ -548,9 +715,9 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
         if command is None:
             raise ValueError(f"Invalid command name: {command_name}")
 
-        return command.make_context_and_run(self, current_app, data)
+        return command.make_context_and_run(discord=self, app=current_app, data=data)
 
-    def run_handler(self, data, *, allow_modal=True):
+    def run_handler(self, data: dict, *, allow_modal: bool = True):
         """
         Run the corresponding custom ID handler given incoming interaction
         data.
@@ -559,6 +726,11 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
         ----------
         data
             Incoming interaction data.
+
+        Returns
+        -------
+        Message
+            The resulting message.
         """
 
         context = Context.from_data(self, current_app, data)
@@ -574,7 +746,7 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
 
         return Message.from_return_value(result)
 
-    def run_autocomplete(self, data):
+    def run_autocomplete(self, data: dict):
         """
         Run the corresponding autocomplete handler given incoming interaction
         data.
@@ -583,6 +755,11 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
         ----------
         data
             Incoming interaction data.
+
+        Returns
+        -------
+        AutocompleteResult
+            The result of the autocomplete handler.
         """
 
         context = Context.from_data(self, current_app, data)
@@ -621,7 +798,35 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
         if not request.json:
             abort(400, "Request JSON required")
 
-    def set_route(self, route, app=None):
+    def handle_request(self):
+        """
+        Verify the signature in the incoming request and return the Message
+        result from the given command.
+
+        Returns
+        -------
+        Message
+            The resulting message from the command.
+        """
+        self.verify_signature(request)
+
+        interaction_type = request.json.get("type")
+        if interaction_type == InteractionType.PING:
+            abort(jsonify({"type": ResponseType.PONG}))
+        elif interaction_type == InteractionType.APPLICATION_COMMAND:
+            return self.run_command(request.json)
+        elif interaction_type == InteractionType.MESSAGE_COMPONENT:
+            return self.run_handler(request.json)
+        elif interaction_type == InteractionType.APPLICATION_COMMAND_AUTOCOMPLETE:
+            return self.run_autocomplete(request.json)
+        elif interaction_type == InteractionType.MODAL_SUBMIT:
+            return self.run_handler(request.json, allow_modal=False)
+        else:
+            raise RuntimeWarning(
+                f"Interaction type {interaction_type} is not yet supported"
+            )
+
+    def set_route(self, route: str, app: Flask = None):
         """
         Add a route handler to the Flask app that handles incoming
         interaction data.
@@ -631,9 +836,9 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
 
         Parameters
         ----------
-        route
+        route: str
             The URL path to receive interactions on.
-        app
+        app: Flask
             The Flask app to add the route to.
         """
 
@@ -642,32 +847,11 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
 
         @app.route(route, methods=["POST"])
         def interactions():
-            self.verify_signature(request)
+            result = self.handle_request()
+            response, mimetype = result.encode()
+            return Response(response, mimetype=mimetype)
 
-            interaction_type = request.json.get("type")
-            if interaction_type == InteractionType.PING:
-                return jsonify({"type": ResponseType.PONG})
-
-            elif interaction_type == InteractionType.APPLICATION_COMMAND:
-                return jsonify(self.run_command(request.json).dump())
-
-            elif interaction_type == InteractionType.MESSAGE_COMPONENT:
-                return jsonify(self.run_handler(request.json).dump_handler())
-
-            elif interaction_type == InteractionType.APPLICATION_COMMAND_AUTOCOMPLETE:
-                return jsonify(self.run_autocomplete(request.json).dump())
-
-            elif interaction_type == InteractionType.MODAL_SUBMIT:
-                return jsonify(
-                    self.run_handler(request.json, allow_modal=False).dump_handler()
-                )
-
-            else:
-                raise RuntimeWarning(
-                    f"Interaction type {interaction_type} is not yet supported"
-                )
-
-    def set_route_async(self, route, app=None):
+    def set_route_async(self, route: str, app: Flask = None):
         """
         Add a route handler to a Quart app that handles incoming interaction
         data using asyncio.
@@ -677,10 +861,10 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
 
         Parameters
         ----------
-        route
+        route: str
             The URL path to receive interactions on.
-        app
-            The Flask app to add the route to.
+        app: Flask
+            The Quart app to add the route to.
         """
 
         if app is None:
@@ -688,36 +872,18 @@ class DiscordInteractions(DiscordInteractionsBlueprint):
 
         if aiohttp is None:
             raise ImportError(
-                "The aiohttp module is required for async usage of this " "library"
+                "The aiohttp module is required for async usage of this library"
             )
 
         @app.route(route, methods=["POST"])
         async def interactions():
-            self.verify_signature(request)
-
-            interaction_type = request.json.get("type")
-            if interaction_type == InteractionType.PING:
-                return jsonify({"type": ResponseType.PONG})
-
-            elif interaction_type == InteractionType.APPLICATION_COMMAND:
-                result = self.run_command(request.json)
-            elif interaction_type == InteractionType.MESSAGE_COMPONENT:
-                result = self.run_handler(request.json)
-            elif interaction_type == InteractionType.APPLICATION_COMMAND_AUTOCOMPLETE:
-                result = self.run_autocomplete(request.json)
-
-            elif interaction_type == InteractionType.MODAL_SUBMIT:
-                result = self.run_handler(request.json, allow_modal=False)
-
-            else:
-                raise RuntimeWarning(
-                    f"Interaction type {interaction_type} is not yet supported"
-                )
+            result = self.handle_request()
 
             if inspect.isawaitable(result):
                 result = await result
 
-            return jsonify(result.dump())
+            response, mimetype = result.encode()
+            return Response(response, mimetype=mimetype)
 
         # Set up the aiohttp ClientSession
 
