@@ -1,4 +1,5 @@
 import dataclasses
+from typing import Optional
 
 from flask_discord_interactions.models.utils import LoadableDataclass
 
@@ -11,7 +12,13 @@ class ChannelType:
     GROUP_DM = 3
     GUILD_CATEGORY = 4
     GUILD_NEWS = 5
-    GUILD_STORE = 6
+    GUILD_STORE = 6  # deprecated
+    ANNOUNCEMENT_THREAD = 10
+    PUBLIC_THREAD = 11
+    PRIVATE_THREAD = 12
+    GUILD_STAGE_VOICE = 13
+    GUILD_DIRECTORY = 14
+    GUILD_FORUM = 15
 
 
 @dataclasses.dataclass
@@ -30,9 +37,18 @@ class Channel(LoadableDataclass):
         The permissions integer of the invoking user in that channel.
     type
         The type of channel.
+    nsfw
+        Whether the channel is age restricted or not.
+    parent_id
+        Category this channel belongs to.
+    thread_metadata
+        Thread-specific fields not needed by other channels.
     """
 
-    id: str = None
-    name: str = None
-    permissions: int = None
-    type: int = None
+    id: str
+    name: str
+    permissions: int
+    type: int
+    nsfw: Optional[bool] = False
+    parent_id: Optional[bool] = None
+    thread_metadata: Optional[dict] = None
