@@ -1,3 +1,5 @@
+import json
+
 from dataclasses import dataclass
 from typing import List, Union
 
@@ -56,15 +58,34 @@ class Modal(LoadableDataclass):
                 raise ValueError("Only Action Row components are supported for Modals")
 
     def dump_components(self):
-        "Returns the message components as a list of dicts."
+        """
+        Returns the message components as a list of dicts.
+
+        Returns
+        -------
+        List[dict]
+            The message components as a list of dicts.
+        """
         return [c.dump() for c in self.components]
 
-    def dump(self):
+    def encode(self, followup: bool = False):
         """
         Return this ``Modal`` as a dict to be sent in response to an
         incoming webhook.
+
+        Parameters
+        ----------
+        followup: bool
+            Whether this is a followup to a previous modal.
+
+        Returns
+        -------
+        str
+            The JSON-encoded modal.
+        str
+            The mimetype of the response (``application/json``).
         """
-        return {
+        payload = {
             "type": ResponseType.MODAL,
             "data": {
                 "custom_id": self.custom_id,
@@ -73,6 +94,4 @@ class Modal(LoadableDataclass):
             },
         }
 
-    def dump_handler(self):
-        "Return a modal in component handlers."
-        return self.dump()
+        return (json.dumps(payload), "application/json")
